#-----------------------------------------------------------------------
# universe.py
#
# Z knihy Sedgewick, Wayne, Dondero,
# http://introcs.cs.princeton.edu/python/code/
# upravil J.Kybic pro použití standardních knihoven, 2017
#-----------------------------------------------------------------------

import sys
from body import Body 
import numpy as np              # Numeric Python
import matplotlib.pyplot as plt # pro zobrazování a grafy

#-----------------------------------------------------------------------

colors='rgbkymc' # sequence of colour codes to use for different bodies

class Universe:

    # Construct a new Universe object by reading a description
    # from the file whose name is filename.

    def __init__(self, filename):
        with open(filename,'rt') as instream:
          n = int(instream.readline())
          radius = float(instream.readline())
          plt.clf()
          plt.axis((-radius,radius,-radius,radius)) # velikost vesmíru
          self._bodies = []
          for i in range(n):
              line=instream.readline().split()
              rx   = float(line[0])
              ry   = float(line[1])
              vx   = float(line[2])
              vy   = float(line[3])
              mass = float(line[4])
              r = np.array([rx, ry])
              v = np.array([vx, vy])
              self._bodies += [Body(r, v, mass,colors[i % len(colors)])]

    # Simulate the passing of dt seconds in self.
    
    def increaseTime(self, dt):
        
        # Initialize the forces to zero.
        n = len(self._bodies)
        f = [ np.zeros(2) for i in range(n) ]
        
        # Compute the forces.
        for i in range(n):
            for j in range(n):
                if i != j:
                    bodyi = self._bodies[i]
                    bodyj = self._bodies[j]
                    f[i] = f[i] + bodyi.forceFrom(bodyj)

        # Move the bodies.
        for i in range(n):
            self._bodies[i].move(f[i], dt)    

    # Draw self to standard draw.
    def draw(self):
        for body in self._bodies:
            body.draw()

#-----------------------------------------------------------------------

# Accept a string 'filename', a float 'dt' and integer 'n' as command-line arguments.
# Simulate the motion in the universe defined by the contents of
# the file with the given filename, increasing time at the rate
# specified by dt for n steps (third

def main():
    filename = sys.argv[1]
    dt = float(sys.argv[2])
    n=int(sys.argv[3])
    universe = Universe(filename)
    plt.figure(1)
    for i in range(n):
        universe.increaseTime(dt)
        universe.draw()
    plt.savefig('figs/'+filename+'-%d.pdf' % n)
    plt.show()
    

if __name__ == '__main__':
    main()

#-----------------------------------------------------------------------
# Příklady volání tohoto skriptu
#
# python3 universe.py 3body.txt 20000 1000
#
# python3 universe.py 4body.txt 20000 1000

